<?php
# database settings managed by Debian configuration
# Do not touch this file

@include '/etc/spotweb/dbconfig.php';
$settings['db']['engine'] = "pdo_$dbtype";
$settings['db']['host'] = $dbserver;
$settings['db']['dbname'] = $dbname;
$settings['db']['user'] = $dbuser;
$settings['db']['pass'] = $dbpass;

