<?php
# database settings managed by Debian configuration
# Do not touch this file

@include '/etc/spotweb/dbconfig.php';
$dbsettings['engine'] = "pdo_$dbtype";
$dbsettings['host'] = $dbserver ?: "localhost";
$dbsettings['dbname'] = $dbname;
$dbsettings['user'] = $dbuser;
$dbsettings['pass'] = $dbpass;

